import httpx
import ipaddress
import execjs
from textual import on
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Markdown, Input, Header, Checkbox, Label, Footer, TabbedContent, TabPane, Select, ListView,ListItem, Button, TextArea
from textual.containers import Horizontal, Vertical, HorizontalScroll
from rapidfuzz import process, fuzz, utils as fuzz_utils

class Rev(Screen):

    BINDINGS = [
        ("escape", "pop_screen", "Close"),
        ("f6", "select_line", "Select Line"),
        ("f7", "select_all", "Select All"),
    ]

    def validate_ip(self, ip: str) -> bool:
        try:
            ipaddress.ip_address(ip)
            return True
        except ValueError:
            return False

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        with HorizontalScroll(id="border"):
            with Vertical(id="ip-container"):
                yield Input(placeholder="IP Address", id="input-ip", restrict=r"^[0-9.]*$")
            with Vertical(id="port-container"):
                yield Input(placeholder="Port", id="input-port", restrict=r"^(6553[0-5]|655[0-2]\d|65[0-4]\d{2}|6[0-4]\d{3}|[1-5]\d{4}|[1-9]\d{0,3}|0)$")
            
            with Vertical(id="listener-selection-container"):
                yield Select([], prompt="Select Listener", id="select-listener")

            with HorizontalScroll(id="listner-container"):
                yield TextArea(id="listen-text")
                
        with HorizontalScroll(id="results-container"):
            with TabbedContent():
                with TabPane("Reverse", id="reverse"):
                    with Horizontal():
                        with Vertical(id="left-bottom"):
                            yield Input(placeholder="Search Name", id="reverse-input-search")
                            yield ListView(id="reverse-shell-list")
                        with Vertical():
                            with Horizontal(id="checkbox"):
                                yield from self.generate_checkboxes('reverse')
                                yield Select([], prompt="Select Shell", id="select-reverse")
                            with HorizontalScroll():
                                yield TextArea(id="results-reverse")
                with TabPane("Bind", id="bind"):
                    with Horizontal():
                        with Vertical(id="left-bottom"):
                            yield Input(placeholder="Search Name", id="bind-input-search")
                            yield ListView(id="bind-shell-list")
                        with Vertical():
                            with Horizontal(id="checkbox"):
                                yield from self.generate_checkboxes('bind')
                            with HorizontalScroll():
                                yield TextArea(id="results-bind")
                with TabPane("MSFVenom", id="msfvenom"):
                    with Horizontal():
                        with Vertical(id="left-bottom"):
                            yield Input(placeholder="Search Name", id="msfvenom-input-search")
                            yield ListView(id="msfvenom-list")
                        with Vertical():
                            with Horizontal(id="checkbox"):
                                yield from self.generate_checkboxes('msfvenom')
                            with HorizontalScroll():
                                yield TextArea(id="results-msfvenom")
                with TabPane("hoaxShell", id="hoax"):
                    with Horizontal():
                        with Vertical(id="left-bottom"):
                            yield Input(placeholder="Search Name", id="hoax-input-search")
                            yield ListView(id="hoax-shell-list")
                        with Vertical():
                            with Horizontal(id="checkbox"):
                                yield from self.generate_checkboxes('hoax')
                            with HorizontalScroll():
                                yield TextArea(id="results-hoax")
                
        

    async def on_mount(self) -> None:
        """Called when app starts."""
        self.title = "Gibme"
        self.sub_title = "Reverse Shell Generator"

        self.ip = '10.10.10.10'
        self.port = '4444'
        self.selected_listener = 'nc'
        self.selected_shell = 'sh'

        self.rev_data = await self.parse_rev_data()
        self.populate_listener_data(self.rev_data)
        self.populate_tabs_shell_list()

        self.populate_tabs_shell_data(['all'], None)
        self.update_listener(self.selected_listener)

        self.active_tab = self.query_one(TabbedContent).active
        self.active_tab = self.active_tab.lower()
        if self.active_tab == 'reverse':
            self.list_id = 'reverse-shell-list'
            self.selected_rev_shell = self.query_one(f'#{self.list_id}', ListView).children[0].id


    async def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == 'copy-listener':
            # get listener text from label and remove Listener: from text
            pyperclip.copy(self.listener_text)


    async def on_checkbox_changed(self) -> None:
        # get checked boxes
        checkboxes = self.query(Checkbox)
        self.checked_boxes = [checkbox.id for checkbox in checkboxes if checkbox.value]

        # get active tab
        self.active_tab = self.query_one(TabbedContent).active

        # update os list to match only os with active tab to tabname and remove tab name
        self.active_tab = self.active_tab.lower()
        self.checked_boxes = [x.replace(f'-{self.active_tab}', '') for x in self.checked_boxes if self.active_tab in x]

        self.populate_tabs_shell_data(self.checked_boxes, None)


    async def on_input_changed(self, event: Input.Changed) -> None:
        # get input event id
        input_id = event.input.id

        # get input value for both ip and port
        if input_id == 'input-ip':
            if self.validate_ip(self.query_one("#input-ip").value):
                self.ip = self.query_one("#input-ip").value
            else:
                self.ip = '10.10.10.10'

        #self.ip = self.query_one("#input-ip").value
        if input_id == 'input-port':
            self.port = self.query_one("#input-port").value

            if self.port == '':
                self.port = '4444'
            elif int(self.port) > 65535:
                self.port = '4444'
            
            if self.ip == '':
                self.ip = '10.10.10.10'

        # fuzz search for shell
        self.active_tab = self.query_one(TabbedContent).active
        self.active_tab = self.active_tab.lower()
        if input_id == f'{self.active_tab}-input-search':
            rev_shell_name = self.query_one(f"#{input_id}").value
            if rev_shell_name == '':
                self.populate_tabs_shell_data(['all'], None)
            else:
                if self.active_tab == 'reverse':
                    choices = self.reverse_shell_list
                elif self.active_tab == 'bind':
                    choices = self.bind_shell_list
                elif self.active_tab == 'msfvenom':
                    choices = self.msfvenom_list
                elif self.active_tab == 'hoax':
                    choices = self.hoax_shell_list
                
                fuzz_result = await self.fuzz_shell(rev_shell_name, choices)
                self.populate_tabs_shell_data(['all'], fuzz_result)


        self.update_listener(self.selected_listener)

        if self.selected_rev_shell:
            self.update_results(self.list_id, self.selected_rev_shell, self.selected_shell)


    async def on_select_changed(self, event: Select.Changed) -> None:
        select_id = event.select.id

        # get selected listener
        if select_id == 'select-listener':
            self.selected_listener = self.query_one('#select-listener', Select).value
            self.update_listener(self.selected_listener)
        elif select_id == 'select-reverse':
            self.selected_shell = self.query_one('#select-reverse', Select).value

            if self.selected_rev_shell:
                self.update_results(self.list_id, self.selected_rev_shell, self.selected_shell)


    async def on_list_view_selected(self, event: ListView.Selected) -> None:
        self.list_id = event.list_view.id
        #self.query_one(Markdown).update(f'{event.item.children[0].id}')
        self.selected_rev_shell = event.item.children[0].name

        if self.selected_rev_shell:
            self.update_results(self.list_id, self.selected_rev_shell, self.selected_shell)

    @on(TabbedContent.TabActivated)
    def clear_input(self) -> None:
        self.active_tab = self.query_one(TabbedContent).active
        self.populate_tabs_shell_data(['all'], None)
        tab_id = ['reverse', 'bind', 'msfvenom', 'hoax']
        for id in tab_id:
            if id != self.active_tab:
                self.query_one(f"#{id}-input-search", Input).value = ''

    def update_listener(self, listener: str) -> None:
        # get match listener data from listener list
        listener_data = [listener for listener in self.rev_data['rsgData']['listenerCommands'] if listener[0] == self.selected_listener][0]

        # edit port and display listener data
        self.listener_text = listener_data[1].replace('{port}', self.port)
        if int(self.port) < 1024:
            self.listener_text = f'sudo {self.listener_text}'

        self.query_one("#listen-text", TextArea).load_text(self.listener_text)


    def update_results(self, list_id: str, rev_shell: str, shell:str) -> None:
        if list_id == 'reverse-shell-list':
            tab = 'reverse'
        elif list_id == 'bind-shell-list':
            tab = 'bind'
        elif list_id == 'msfvenom-list':
            tab = 'msfvenom'
        elif list_id == 'hoax-shell-list':
            tab = 'hoax'

        if tab == 'reverse':
            shell_data = [shell for shell in self.rev_data['reverseShellCommands'] if shell['name'] == rev_shell][0]['command']
        elif tab == 'bind':
            shell_data = [shell for shell in self.rev_data['bindShellCommands'] if shell['name'] == rev_shell][0]['command']
        elif tab == 'msfvenom':
            shell_data = [shell for shell in self.rev_data['msfvenomCommands'] if shell['name'] == rev_shell][0]['command']
        elif tab == 'hoax':
            shell_data = [shell for shell in self.rev_data['hoaxShellCommands'] if shell['name'] == rev_shell][0]['command']

        # replace {ip} {port} {shell} with user input if there exists
        if '{ip}' in shell_data:
            shell_data = shell_data.replace('{ip}', self.ip)
        if '{port}' in shell_data:
            shell_data = shell_data.replace('{port}', self.port)
        if '{shell}' in shell_data:
            shell_data = shell_data.replace('{shell}', shell)

        self.query_one(f"#results-{tab}", TextArea).clear()
        self.query_one(f"#results-{tab}", TextArea).load_text(f'{shell_data}')


    def generate_checkboxes(self, tab_name: str) -> None:
        yield Checkbox("All", "True", id=f"all-{tab_name}")
        yield Checkbox("Linux", id=f"linux-{tab_name}")
        yield Checkbox("MacOSX", id=f"mac-{tab_name}")
        yield Checkbox("Windows", id=f"windows-{tab_name}")


    def populate_listener_data(self, data: list) -> None:
        listener_select_list = []
        for listener in data['rsgData']['listenerCommands']:
            listener_select_list.append((listener[0], listener[0]))

        self.query_one('#select-listener', Select).set_options(listener_select_list)
        self.query_one('#select-listener', Select).value = listener_select_list[0][0]


    def populate_tabs_shell_list(self) -> None:
        shell_select_list = []
        for shell in self.rev_data['rsgData']['shells']:
            shell_select_list.append((shell, shell))

        self.query_one('#select-reverse', Select).set_options(shell_select_list)
        self.query_one('#select-reverse', Select).value = shell_select_list[0][0]


    def _update_list_view(self, list_id: str, shell_list: list) -> None:
        list_view = self.query_one(f'#{list_id}', ListView)
        list_view.clear()
        list_view.extend([ListItem(Label(shell, name=shell)) for shell in shell_list])
        list_view.index = 0


    def populate_tabs_shell_data(self, os: list, fuzz_list: list) -> None:
        self.reverse_shell_list = self._get_shell_list('reverseShellCommands', os)
        self.bind_shell_list = self._get_shell_list('bindShellCommands', os)
        self.msfvenom_list = self._get_shell_list('msfvenomCommands', os)
        self.hoax_shell_list = self._get_shell_list('hoaxShellCommands', os)

        # Reverse Shell
        if fuzz_list:
            self._update_list_view('reverse-shell-list', fuzz_list)
        else:
            self._update_list_view('reverse-shell-list', self.reverse_shell_list)

        # Bind Shell
        
        if fuzz_list:
            self._update_list_view('bind-shell-list', fuzz_list)
        else:
            self._update_list_view('bind-shell-list', self.bind_shell_list)

        # MSFVenom
        
        if fuzz_list:
            self._update_list_view('msfvenom-list', fuzz_list)
        else:
            self._update_list_view('msfvenom-list', self.msfvenom_list)

        # HoaxShell
        
        if fuzz_list:
            self._update_list_view('hoax-shell-list', fuzz_list)
        else:
            self._update_list_view('hoax-shell-list', self.hoax_shell_list)


    def _get_shell_list(self, shell_type: str, os: list = None) -> list:
        shell_list = []
        for shell in self.rev_data[shell_type]:
            if os and 'all' not in os:
                if any(os_item in shell['meta'] for os_item in os):
                    shell_list.append(shell['name'])
            elif 'all' in os:
                shell_list.append(shell['name'])
            else:
                shell_list = []
        return shell_list


    async def fuzz_shell(self, shell_name: str, choices: list) -> list:
        shell_name = shell_name.lower()
        results = process.extract(
                shell_name, 
                choices, 
                scorer=fuzz.QRatio,
                limit=10,
                score_cutoff=40,
                processor=fuzz_utils.default_process,)
        if results:
            highest_similarity = max(results, key=lambda x: x[1])
            if highest_similarity[1] > 90:
                return [highest_similarity[0]]
            else:
                return [result[0] for result in results]
        else:
            return None
            

    async def parse_rev_data(self) -> list:
        try:
            runtime = execjs.get()
            async with httpx.AsyncClient() as client:
                response = await client.get("https://raw.githubusercontent.com/0dayCTF/reverse-shell-generator/main/js/data.js")
                context = runtime.compile(response.text)
                commands = {
                'reverseShellCommands': context.eval('reverseShellCommands'),
                'bindShellCommands': context.eval('bindShellCommands'),
                'msfvenomCommands': context.eval('msfvenomCommands'),
                'hoaxShellCommands': context.eval('hoaxShellCommands'),
                'rsgData': context.eval('rsgData')
            }
            return commands

        except Exception as e:
            self.query_one("#results", Markdown).update(f"**Error getting reverse shell data**\n\nError: {e}")


from __future__ import annotations
from textual.app import App, ComposeResult
from textual.containers import Horizontal
from textual.widget import Widget
from textual.widgets import Footer, Label, Markdown
from textual.binding import Binding
from textual.screen import Screen

from screens.about_screen import About
from screens.gtfo_screen import GTFO
from screens.lol_screen import LOL
from screens.tldr_screen import TLDR
from screens.rev_screen import Rev
from screens.cve_screen import CVE
from screens.cheat_screen import Cheat

class GibmeHeader(Widget):
    
    def compose(self) -> ComposeResult:
        with Horizontal():
            title = Label(self.app.title, id="app_title")
            yield title

class Gibme(Screen):
    BINDINGS = [
        Binding("g", "push_screen('gtfo')", "GTFO_Bin"),
        Binding("l", "push_screen('lol')", "LOLBAS"),
        Binding("r", "push_screen('rev')", "Reverse shell"),
        Binding("t", "push_screen('tldr')", "TLDR"),
        Binding("s", "push_screen('cve')", "CVE Search"),
        Binding("c", "push_screen('cheat')", "Cheat Sheet"),
        Binding("q", "quit", "Quit"),
        Binding("question_mark", "push_screen('about')", "About", key_display="?"),
    ]
    HELP_MARKDOWN = """\
Here are the key bindings for this application:

- **g:** GTFO_Bin
- **l:** LOLBAS
- **r:** Reverse shell
- **t:** TLDR
- **s:** CVE Search
- **c:** Cheat Sheet
- **q:** Quit
- **?:** About
"""
    
    def compose(self) -> ComposeResult:
        yield GibmeHeader()
        yield Footer()
        yield Markdown(self.HELP_MARKDOWN)

    def action_tldr() -> None:
        pass

    def action_quit() -> None:
        pass

class GibmeApp(App[None]):

    CSS_PATH = "gibme.tcss"
    TITLE = "Gibme TUI"

    SCREENS = {
        "about": About,
        "gtfo": GTFO,
        "lol": LOL,
        "tldr": TLDR,
        "rev": Rev,
        "cve": CVE,
        "cheat": Cheat,
    }

    def on_mount(self) -> None:
        self.push_screen(Gibme())
    
def main():
    app = GibmeApp()
    app.run()

if __name__ == "__main__":
    main()


import httpx
import asyncio
import json

from bs4 import BeautifulSoup


class CVE:
    def __init__(self, cve_name: str = None, search_keyword: str = None):
        self.cve_name = cve_name
        self.search_keyword = search_keyword
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/113.0"
        }

    def get_cve_poc(self) -> dict:
        url = "https://raw.githubusercontent.com/trickest/cve/main/references.txt"
        with httpx.Client() as client:
            response = client.get(url, headers=self.headers)
            data = response.text if response.status_code == 200 else "Error"
            for line in data.split("\n"):
                name, link = line.split(" - ")
                # dump into dict and return dict = {"name": "link"}
                yield {name: link}

    def search_cve(self):
        url = "https://cve.mitre.org/cgi-bin/cvekey.cgi?keyword={}"

    def get_cve(self):
        url = "https://cve.mitre.org/cgi-bin/cvename.cgi?name={}"



import httpx
from textual import work
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Markdown, Input, Header, Footer
from textual.containers import VerticalScroll
from yaml import load_all, SafeLoader
from bs4 import BeautifulSoup
from rapidfuzz import process, fuzz, utils as fuzz_utils

class LOL(Screen):

    BINDINGS = [("escape", "pop_screen", "Close")]

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        yield Input(placeholder="Search for LOL Bins")
        with VerticalScroll(id="results-container"):
            yield Markdown(id="results")

    async def on_mount(self) -> None:
        """Called when app starts."""
        self.query_one(Input).focus()
        self.title = "Gibme"
        self.sub_title = "LOL Bins"
        self.exe_list = await self.get_exe()

    async def on_input_changed(self, message: Input.Changed) -> None:
        """A coroutine to handle a text changed message."""
        if message.value:
            fuzz_result = await self.fuzz_exe(message.value)
            if len(fuzz_result) == 0:
                self.query_one("#results", Markdown).update(f"**Not Found**")
            elif len(fuzz_result) == 1:
                if '.exe' in fuzz_result[0]:
                    exe_name = fuzz_result[0].replace(".exe", "")
                self.lolbas_info(exe_name)
            else:
                self.query_one("#results", Markdown).update(self.fuzz_result_markdown(fuzz_result))
        else:
            # Clear the results
            self.query_one("#results", Markdown).update("")

    @work(exclusive=True)
    async def lolbas_info(self, exe_name: str) -> None:
        self.query_one("#results", Markdown).update(f'**{exe_name}**')
        raw_url = f"https://raw.githubusercontent.com/LOLBAS-Project/LOLBAS/master/yml/OSBinaries/{exe_name}.yml"
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(raw_url)
                markdown = self.make_word_markdown(response)
                self.query_one("#results", Markdown).update(markdown)
            except Exception:
                self.query_one("#results", Markdown).update(response.text)

    def make_word_markdown(self, response: httpx.Response) -> str:
        """Make a markdown string for a word."""
        if response.status_code == 404:
            return "### Not Found"

        data = list(load_all(response.text, Loader=SafeLoader))[0]
        markdown = f"# **{data['Name']}**\n\n"
        markdown += f"**Description:** *{data['Description']}*\n\n"
        markdown += f"**Author:** *{data['Author']}*\n\n"
        markdown += f"**Created:** *{data['Created']}*\n\n"

        for command in data["Commands"]:
            markdown += f"- **Command:**\n\n```\n{command['Command']}\n```\n\n"
            markdown += f"**Description:** *{command['Description']}*\n\n"
            markdown += f"**Usecase:** *{command['Usecase']}*\n\n"
            markdown += f"**Category:** *{command['Category']}*\n\n"
            markdown += f"**Privileges:** *{command['Privileges']}*\n\n"
            markdown += f"**MitreID:** *{command['MitreID']}*\n\n"
            markdown += f"**OperatingSystem:** *{command['OperatingSystem']}*\n\n"
            markdown += "\n---\n\n"

        return markdown
    
    async def get_exe(self) -> list:
        URL = "https://lolbas-project.github.io/"
        exe = {}
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(URL)
                soup = BeautifulSoup(response.text, "lxml")
                tds = soup.find_all("a", class_="bin-name")
                for i in tds:
                    exe[i.text] = i["href"][8:][:-1]

                return exe
            except Exception as e:
                self.query_one("#results", Markdown).update(f"**Check your internet connection - Error: Cound not connect to https://lolbas-project.github.io/ error: {e}**")

    async def fuzz_exe(self, exe_name: str) -> list:
        if '.exe' not in exe_name:
            exe_name = f'{exe_name}.exe'
        choices = list(self.exe_list.keys())
        results = process.extract(
            exe_name, 
            choices, 
            scorer=fuzz.ratio,
            limit=10,
            score_cutoff=70,
            processor=fuzz_utils.default_process,)
        if results:
            highest_similarity = max(results, key=lambda x: x[1])
            if highest_similarity[1] > 90:
                return [highest_similarity[0]]
            else:
                return results
        else:
            return results

    def fuzz_result_markdown(self, results: list) -> str:
        markdown = ""
        for result in results:
            markdown += f"**{result[0]}**\n\n"
        return markdown